GUI.Plugin.DQMHeaderRow = new function() {
  var _gui = null;
  var _data = null;
  var _header = document.getElementById('header');
  var _service = null;
  var _runGroup = null;
  var _runNr = null;
  var _lumiNr = null;
  var _eventNr = null;
  var _workspace = null;
  var _pageNr = null;
  var _npage = 1;
  var _npages = 1;

  this.onresize = function()
  {
    var b = _header.clientHeight;
    for (var i = _header.firstChild; i; i = i.nextSibling)
    {
     var bottom = i.offsetTop + i.clientHeight;
     if (bottom > b)
       b = bottom;
    }

    var top = _header.nextSibling.offsetTop;
    var newtop = b+15;
    if (top != newtop)
      _header.nextSibling.style.top = newtop + 'px';
  }

  this.heading = function(panel, label, content)
  {
    var hdiv = document.createElement("div");
    _header.insertBefore(hdiv, _header.lastChild);
    hdiv.className = 'heading' + (panel ? " with-panel" : "");
    hdiv.innerHTML = ("<div class='label'>" + label + "</div><h2>" + content + "</h2>"
		      + (panel ? "<div class='panel'></div>" : ""));
    return hdiv.childNodes[1];
  }

  this.attach = function(gui)
  {
    var panel;
    _gui = gui;
    _service = this.heading(true, "DQM service",
			    "<span>_</span>:").firstChild;
    _service.parentNode.nextSibling.innerHTML +=
      "<table id='hdr-service'><tbody></tbody></table>";

    _runNr = this.heading(true, "Run",
			  "<span>_</span><span class='delimiter'>"
			  + "&nbsp;&nbsp;.</span>").firstChild;
    _lumiNr = this.heading(false, "LS",
			   "<span>_</span><span class='delimiter'>"
			   + "&nbsp;&nbsp;.</span>").firstChild;
    _eventNr = this.heading(false, "Event",
			    "<span>_</span><span class='delimiter'>"
			    + "&nbsp;&nbsp;.</span>").firstChild;
    _runNr.parentNode.nextSibling.innerHTML += "<table id='hdr-runnr' style='font-size:.9em'><tbody>"
      +"<tr><td><input style='font-size:100%' type='text' onKeyUp='return GUI.Plugin.DQMHeaderRow.runNrKeyPress(event)'/>"
          +" &nbsp; &nbsp; <span style='font-size: small'></span></td></tr>"
        +"<tr><td><div></div></td></tr>"
        +"<tr><td><div></div></td></tr>"
      +"</tbody></table>";

    _workspace = this.heading(true, "Workspace",
			      "<span>_</span>,").firstChild;
    _workspace.parentNode.nextSibling.innerHTML +=
      "<table id='hdr-workspace' style='font-size:.9em'><tbody valign='top'></tbody></table>";

    _pageNr = this.heading(true, "Page",
			   "<span class='delimiter'><a href='#'"
			   + " onclick='return GUI.Plugin.DQMHeaderRow.movePage(-1)'>"
			   + "<img width='20' height='15' src='"
			   + FULLROOTPATH + "/static/left.gif'/></a></span>"
			   + "<span>1 / 1</span>"
			   + "<span class='delimiter'><a href='#'"
			   + " onclick='return GUI.Plugin.DQMHeaderRow.movePage(1)'>"
			   + "<img width='20' height='15' src='"
			   + FULLROOTPATH + "/static/right.gif'/></a></span>").childNodes[1];
    _pageNr.parentNode.nextSibling.innerHTML +=
      ("Go to page <input style='font-size:100%' id='hdr-pagenr' size='4' type='text'"
       + " onchange='return GUI.Plugin.DQMHeaderRow.setPage(this.value)' />");
  }

  this.detach = function()
  {
    _header.removeChild(_service.parentNode.parentNode);
    _header.removeChild(_runNr.parentNode.parentNode);
    _header.removeChild(_lumiNr.parentNode.parentNode);
    _header.removeChild(_eventNr.parentNode.parentNode);
    _header.removeChild(_workspace.parentNode.parentNode);
    _header.removeChild(_pageNr.parentNode.parentNode);
    _service = null;
    _runNr = null;
    _lumiNr = null;
    _eventNr = null;
    _workspace = null;
    _pageNr = null;
  }

  this.movePage = function(by)
  {
    var page = _npage + by;
    if (page > _npages) page = 1;
    if (page < 1) page = _npages;
    _gui.makeCall(_url() + "/setPage?page=" + page);
    return false;
  }

  this.setPage = function(choice)
  {
    _gui.makeCall(_url() + "/setPage?page=" + encodeURIComponent(choice));
    return false;
  }

  this.setWorkspace = function(choice)
  {
    _gui.makeCall(_url() + "/workspace?name=" + encodeURIComponent(choice));
    return false;
  }

  this.setRun = function(choice)
  {
    _gui.makeCall(_url() + "/setRun?n=" + encodeURIComponent(choice));
    return false;
  }

  this.update = function(data)
  {
    _data = data;
    _service.innerHTML = data.service;
    _workspace.innerHTML = data.workspace;
    _runNr.innerHTML = data.run;
    _lumiNr.innerHTML = data.lumi;
    _eventNr.innerHTML = data.event;
    _pageNr.innerHTML = data.page + " / " + data.pages;

    var title = data.service + " - " + data.workspace + " - " + HEADING;
    if (document.title != title)
      document.title = title;

    document.getElementById('hdr-pagenr').value = data.page;

    var table, row, cell;
    table = document.getElementById('hdr-service').firstChild;
    for (var i = 0, e = data.services.length; i < e; ++i)
    {
      var item = data.services[i];
      row = i < table.rows.length ? table.rows[i] : table.insertRow(-1);
      cell = row.cells.length ? row.cells[0] : row.insertCell(-1);

      var text;
      if (item.title != data.service)
	text = "<a href='" + item.href + "'>" + item.title + "</a>";
      else
	text = item.title;

      if (cell.innerHTML != text)
	cell.innerHTML = text;
    }

    var category = null;
    table = document.getElementById('hdr-workspace').firstChild;
    row = table.rows.length ? table.rows[0] : table.insertRow(-1);
    data.workspaces.sort(function(a, b) {
      if (a.rank != b.rank)
        return a.rank - b.rank;
      else if (a.title < b.title)
        return -1;
      else if (a.title > b.title)
        return 1;
      else
        return 0;
    });
    for (var c = 0, i = 0, e = data.workspaces.length; i < e; ++i)
    {
      var item = data.workspaces[i];
      if (item.category != category)
      {
        category = item.category;
        cell = c < row.cells.length ? row.cells[c] : row.insertCell(-1);
	cell.innerHTML = "<b style='color:#722'>" + item.category + "</b><br />";
	++c;
      }

      var text;
      if (item.title != data.workspace)
	text = ("<a href='#' onclick=\"return GUI.Plugin.DQMHeaderRow"
		+ ".setWorkspace('" + item.label + "')\">" + item.title
		+ "</a>");
      else
	text = item.title;

      if (i < e-1)
        text += "<br />";

      cell.innerHTML += text;
    }

    this.runNrUpdate(data.runs);

    _npage = data.page;
    _npages = data.pages;
    this.onresize();
  }
  
  this.runNrData = {
    _input:         null,
    _info:          null,
    _header:        null,
    _body:          null,
    _maxHeaderCols: 5,
    _maxBodyRows:   2,
    _maxBodyCols:   10,
    _maxIBodyRuns:   null,
    _runList:       [],
    _index:         null,
    _keyword:       null,
    _matches:       0,
    _exactMatch:    false,
    _isFirstPage:   true,
    _lastkwd:       null
  };
  
  this.runNrUpdate = function (list)
  {
    var r = this.runNrData;
    if (r._input == null)
    {
      var _runNrTable = document.getElementById("hdr-runnr").firstChild;
      r._input  = _runNrTable.rows[0].cells[0].firstChild;
      r._info   = _runNrTable.rows[0].cells[0].lastChild;
      r._header = _runNrTable.rows[1].cells[0].firstChild;
      r._body   = _runNrTable.rows[2].cells[0].firstChild;
      r._maxIBodyRuns = r._maxBodyCols * r._maxBodyRows;
    }
    if (list != undefined)
      r._runList = list;
    r._keyword = r._input.value.replace(/\s/g, "");
    r._hasParent = (r._keyword.length != 0);
    this.runNrIndexer();
    this.runNrDrawPanel();
    return false;
  }
  
  this.runNrKeyPress = function (evt)
  {
    var key = null;
    var r   = this.runNrData;
    if (evt.which) key = evt.which;
    else if (window.event) key = evt.keyCode;
    if (key == 13 && r._exactMatch)
    {
      GUI.Plugin.DQMHeaderRow.setRun(r._input.value.replace(/\s/g, ""))
    }
    else
    {
      r._keyword = r._input.value.replace(/\s/g, "");
      r._hasParent = (r._keyword.length != 0);
      this.runNrIndexer();
      this.runNrDrawPanel();
    }

  }
  
  this.runNrClick = function (kw)
  {
    var r = this.runNrData;
    r._lastkwd = r._keyword;
    r._keyword = (kw == undefined) ? "" : kw.toString();
    if (r._lastkwd != kw)
      r._isFirstPage = true;
    r._hasParent   = (r._keyword.length != 0);
    r._input.value = r._keyword;
    this.runNrIndexer();
    this.runNrDrawPanel();
    return false;
  }
    
  this.runNrNextPage = function()
  {
    this.runNrData._isFirstPage = (!this.runNrData._isFirstPage);
    this.runNrDrawPanel();
    return false;
  }
  
  this.runNrIndexer = function ()
  {
    var r = this.runNrData;
    var kw = r._keyword;
    var lst = r._runList;
    var idx = r._index = {};
    r._matches = 0;
    r._exactMatch = null;
    var rx  = new RegExp("^(" + kw + ")(.)");
    for (var i = 0; i < lst.length; ++i)
    {
      var str = lst[i].toString();
      var m   = rx.exec(str);
      if (m)
      {
        if (! idx[m[2]])
          idx[m[2]] = [];
        idx[m[2]].push(str);
        ++r._matches;
      }
      else if (kw == str)
        r._exactMatch = str;
    }
  }
  
  this.runNrHighLight = function (ch, isOn)
  {
    var color = (isOn) ? "#FFFF00" : "";
    var runs = document.getElementsByName("runset"+ch);
    for (var i = 0; i < runs.length; ++i)
      runs[i].style.backgroundColor = color;
  }
  
  this.runNrDrawPanel = function ()
  {
    var r = this.runNrData;
    var idx = r._index;
    var kw  = r._keyword;
    
    // result message of query
    var msg = "";
    if (kw.length == 0)
      msg += r._matches+" runs, ";
    else
    {
      if (r._exactMatch)
        msg += "[" + r._exactMatch + "] is an exact match, ";
      if (r._matches)
        msg += " " + r._matches.toString() + " match, ";
      if (!r._exactMatch && r._matches == 0)
        msg = "no runs match, ";
    }

    // initialize header & body html text
    var headerHTML = "";
    var bodyHTML   = "";
    var arrowUp    = "";
    var arrowLeft  = "<img src='"+FULLROOTPATH+"/static/runleft.gif'/>";
    var arrowRight = "<img src='"+FULLROOTPATH+"/static/runright.gif'/>";
    var page       = "1/1";
    
    // Up arrow
    if (r._hasParent)
      headerHTML += "<td><a href='#' "
        + "onclick='return GUI.Plugin.DQMHeaderRow.runNrClick("
        + kw.substring(0, kw.length-1) + ")'>"
        + "<img src='"+FULLROOTPATH+"/static/runup.gif'/></a><td>";

    // L and R arrow
    var nIdx = 0;
    for (i in idx) nIdx++;
    if (r._isFirstPage)
    {
      if (nIdx > r._maxHeaderCols)
      {
        arrowRight =
          "<a href='#' onclick='return GUI.Plugin.DQMHeaderRow.runNrNextPage()'>"
          + arrowRight + "</a>";
        page = "1/2";
      }
    }
    else
    {
      arrowLeft =
        "<a href='#' onclick='return GUI.Plugin.DQMHeaderRow.runNrNextPage()'>"
        + arrowLeft + "</a>";
      page = "2/2";
    }
    msg += arrowLeft + "<span style='font-weight: 800; font-size: 120%'>"
      + page + "</span>" + arrowRight;
    
    // partially matched runNr drawing
    var maxElements, iBody;
    var toskip = (r._isFirstPage) ? 0 : r._maxHeaderCols;
    var toshow = r._maxHeaderCols;
    for (var ch = '9'; ch >= '0'; --ch)
    {
      if (idx[ch])
      {
        if ( toskip-- > 0 ) continue;
        if ( toshow-- == 0 ) break;
        
        var len = idx[ch].length;
        if (len > 1) headerHTML +=
          "<td onMouseOver='GUI.Plugin.DQMHeaderRow.runNrHighLight(" + ch + ", true)'"
          + " onMouseOut='GUI.Plugin.DQMHeaderRow.runNrHighLight(" + ch + ", false)'"
          + " style='font-weight: 800; background-color: #eeeeff; width: 7em'>"
          + "<a href='#' "
          + "onClick='return GUI.Plugin.DQMHeaderRow.runNrClick("+kw+ch+")'>"
          + kw + ch + "...</a> "
          + "<span style='font-size: 80%'>(" + len + ")</span></td>";
        
        iBody = "<tr><td colspan='" + r._maxBodyCols + "' "
          + "style='border-bottom: 2px solid #ffddcc; height: 4px;'></td></tr>";
        maxElements = (len > r._maxIBodyRuns) ? (r._maxIBodyRuns - 3) : len;
        for (var i = 0; i < maxElements; ++i)
        {
          iBody += "<td><a href='#' "
            + "onclick='return GUI.Plugin.DQMHeaderRow.setRun(" + idx[ch][i] + ")'>"
            + "<span name='runset" + ch + "'>" + kw + ch + "</span>"
            + idx[ch][i].substring(kw.length+1) + "</a></td>";
        
          if (i%10 == 9 && maxElements > 10)
            iBody += "</tr><tr>";
        }
        if (len > r._maxIBodyRuns)
          iBody +=
            "<td colspan='"+3+"' style='font-size: 80%; color: #cccccc'>"
            + (len - (r._maxIBodyRuns - 3)) + " more...</td>";
        
        bodyHTML += "<tr>" + iBody + "</tr>";
      }
    }
    bodyHTML =
      "<tr><td colspan='"+r._maxHeaderCols+"'>"
      +"<a href='#' onclick='return GUI.Plugin.DQMHeaderRow.setRun(\"Live\")'>"
      +"Live</a></td></tr>" + bodyHTML;
    r._info.innerHTML   = msg;
    r._header.innerHTML = "<table><tr>" + headerHTML + "</tr></table>";
    r._body.innerHTML   = "<table>" + bodyHTML + "</table>";
  }
  return this;
}();

//////////////////////////////////////////////////////////////////////
// Core objects: selector.
GUI.Plugin.DQMList = new function()
{
  var _SIZES		 = [ "XS", "S", "M", "L", "XL" ];
  var _gui               = null;
  var _selector          = this;
  var _menuObjs          = null;
  var _menuObjSize	 = null;
  var _menuObjsDataset   = null;
  var _menuObjsStep      = null;
  var _menuObjsFilter    = null;
  var _menuObjsQuick     = null;
  var _menuObjsData      = null;
  var _menuObjsSearch    = null;
  var _menuObjsSearchRes = null;
  var _sidebar           = document.getElementById('sidebar');
  var _canvas            = document.getElementById('canvas');
  var _click		 = { event: null, timeout: null, timeClick: 0, timeDoubleClick: 0 };
  var _searchDelay;
  var _searchString;

  this.attach = function(gui)
  {
    _gui = gui;
    _sidebar.style.display = '';
    _canvas.style.left = '15.7em';

    _menuObjs = _gui.buildLeftSide('m-objects', 'Objects').childNodes[1];
    _menuObjs.innerHTML = 
      "<table width='100%'><thead"
      + "><tr><td>Dataset</td><td>"
        + "<select style='font-size:100%' id='m-list-dataset' onchange='GUI.Plugin.DQMList.dataset(this.value)'>"
	+ "</select></td></tr"
      + "><tr><td>Step</td><td>"
        + "<select style='font-size:100%' id='m-list-step' onchange='GUI.Plugin.DQMList.step(this.value)'>"
	+ "</select></td></tr"
      + "><tr><td>Filter</td><td>"
        + "<select style='font-size:100%' id='m-list-filter' onchange='GUI.Plugin.DQMList.filter(this.value)'>"
	+ "<option value='all'>Show all</option>"
	+ "<option value='alarms'>Show alarms</option>"
	+ "<option value='nonalarms'>Show non-alarms</option>"
	+ "</select></td></tr"
      + "><tr><td>Search</td><td><input style='font-size:100%' id='m-list-search' type='text' style='width:90%'/></td></tr"
      + "><tr><td>&nbsp;</td><td id='m-list-searchres'>&nbsp;</td></tr"
      + "></thead><tbody id='m-list-quick'/><tbody id='m-list-data'/></table>";

    _menuObjsQuick = document.getElementById('m-list-quick');
    _menuObjsData = document.getElementById('m-list-data');
    _menuObjsDataset = document.getElementById('m-list-dataset');
    _menuObjsStep = document.getElementById('m-list-step');
    _menuObjsFilter = document.getElementById('m-list-filter');
    _menuObjsSearch = document.getElementById('m-list-search');
    _menuObjsSearchRes = document.getElementById('m-list-searchres');
    _menuObjsSearch.onblur = function(e) {
      clearInterval(_searchDelay);
      _searchDelay = null;
      _selector.searchBackground();
    }
    _menuObjsSearch.onfocus = function(e) {
      clearInterval(_searchDelay);
      _searchDelay = setInterval(_selector.searchBackground, 1000);
    }
  }

  this.detach = function()
  {
    var menutop = _menuObjs.parentNode.parentNode;
    menutop.removeChild(_menuObjs.parentNode);
    _sidebar.style.display = 'none'
    _canvas.style.left = '0em';
  }

  this.dataset = function(choice)
  {
    _gui.makeCall(_url() + "/setDataset?n=" + encodeURIComponent(choice));
    return false;
  }

  this.step = function(choice)
  {
    _gui.makeCall(_url() + "/setStep?n=" + encodeURIComponent(choice));
    return false;
  }

  this.filter = function(choice)
  {
    _gui.makeCall(_url() + "/setFilter?n=" + encodeURIComponent(choice));
    return false;
  }

  this.searchBackground = function()
  {
    if (_searchString != _menuObjsSearch.value)
      _selector.search();
  }

  this.search = function()
  {
    _gui.makeCall(_url() + "/setSearch?rx=" + encodeURIComponent(_menuObjsSearch.value));
    _menuObjsSearchRes.innerHTML = "(searching)";
    _searchString = _menuObjsSearch.value;
  }

  this.rebase = function(name)
  {
    _gui.makeCall(_url() + "/setRoot?n=" + encodeURIComponent(name));
    return false;
  }

  this.onRowSelect = function(e)
  {
    var cell = (e.srcElement || e.target);
    while (cell.tagName != 'TD')
      if (cell.tagName == 'A' && cell.getAttribute('dqmto') != null)
	return _selector.rebase(cell.getAttribute('dqmto'));
      else
        cell = cell.parentNode;

    var row = cell.parentNode;
    var dqmname = row.getAttribute('dqmname');
    for (var r = row.parentNode.firstChild; r; r = r.nextSibling)
      _setclass(r.firstChild,
		r.firstChild.className.replace(/ MEispicked/, '')
		+ ((r == row) ? " MEispicked" : ""));
    _gui.makeCall(_url() + "/setPicked?n=" + encodeURIComponent(dqmname));
    return false;
  }

  this.onRowFocus = function(e)
  {
    var cell = (e.srcElement || e.target);
    while (cell.tagName != 'TD')
      if (cell.tagName == 'A')
	return false;
      else
	cell = cell.parentNode;

    var row = cell.parentNode;
    var dqmname = row.getAttribute('dqmname');
    if (cell.className.indexOf("MEisdir") >= 0)
    {
      if (cell.innerHTML.indexOf("Quick collection") >= 0)
        _selector.onRowSelect(e);
      else
        _selector.rebase(dqmname);
    }
    else
      _gui.makeCall(_url() + "/setFocus?n=" + encodeURIComponent(dqmname));

    return false;
  }

  this.updateTreeRow = function(obj, depth, x, dotop)
  {
    if (! x.row)
    {
      x.row = document.createElement("tr");
      x.row.appendChild(document.createElement("td"));
      x.row.firstChild.colSpan = 2;
      x.root.appendChild (x.row);
      x.row.className = "MErow";
      x.row.ondblclick = x.row.onclick = function(e) {
	return _dblclick(e || window.event, _click, _selector.onRowSelect, _selector.onRowFocus); };
    }

    if (x.row.getAttribute('dqmname') != obj.name)
      x.row.setAttribute('dqmname', obj.name);

    var hasalarm = x.row.className.indexOf("MEisalarm");
    if (obj.nalarm > 0 && hasalarm < 0)
      x.row.className += " MEisalarm";
    else if (! obj.nalarm && hasalarm >= 0)
      x.row.className = x.row.className.replace(/ MEisalarm/, "");

    var name = "";
    if (depth == 0 && dotop)
    {
      name = "<a href='#' onclick='return false' dqmto=''>(Top)</a> ";
      if (obj.name != "")
      {
	var from = 0;
	while (from < obj.name.length)
	{
	  var to = obj.name.indexOf("/", from);
	  if (to == -1) to = obj.name.length;

	  name += " / ";
	  if (to < obj.name.length || obj.isdir)
	    name += "<a href='#' onclick='return false' dqmto='"
		    + obj.name.substr(0, to) + "'>"
		    + obj.name.substr(from, to-from)
		    + "</a>";
	  else
	    name += obj.name.substr(from, to-from);

	  from = to+1;
	}
      }
    }
    else if (depth == 0 && !dotop && obj.isdir)
      name = obj.name;
    else
      name = obj.name.replace(/.*\//, "");

    _setvaltitle(x.row.firstChild,
      "ME" + (obj.nlive ? "" : " MEdead")
	   + (obj.isdir ? " MEisdir" : "")
	   + (obj.isdir && depth < 2 ? " MEisopen" : "")
	   + (obj.ispicked ? " MEispicked" : ""),
      "<span style='margin-left: " + depth + "em'>"
      + "<span style='display:-moz-inline-box;display:inline-block;width:1em'>"
      + (obj.isdir ? (depth < 2 ? "-" : "+") : "&nbsp;")
      + "</span>"
      + name
      + "</span>",
      obj.name + ", " + (obj.isdir ? obj.nleaves + " objects, " : "")
      + (obj.nalarm > 1 ? obj.nalarm + " alarms"
	 : obj.nalarm ? (obj.isdir ? "1 alarm" : "has alarm")
	 : (obj.isdir ? "no alarms" : "no alarm")));

    x.nrow++;
    x.row = x.row.nextSibling;
    for (var i = 0, e = obj.kids.length; i < e; ++i)
      this.updateTreeRow(obj.kids[i], depth+1, x, dotop);
  }

  this.updateSelect = function(sel, data, call)
  {
    var opt = sel.firstChild;
    var selectedIndex = -1;
    for (var i = 0, e = data.length; i < e; ++i)
    {
      if (! opt)
	sel.appendChild (opt = document.createElement("option"));

      if (opt.innerHTML != data[i].name)
      {
	opt.innerHTML = data[i].name;
	opt.value = data[i].name;
      }

      if (data[i].selected)
	selectedIndex = i;

      opt = opt.nextSibling;
    }

    while (opt)
    {
      var node = opt;
      opt = opt.nextSibling;
      sel.removeChild(node);
    }

    if (selectedIndex == -1 && data.length > 0)
    {
      _gui.makeCall(_url() + call + encodeURIComponent(data[0].name))
      selectedIndex = 0;
    }

    if (sel.selectedIndex != selectedIndex)
      sel.selectedIndex = selectedIndex;
  }

  this.update = function(data)
  {
    // Update the object tree.
    var x = { root: _menuObjsQuick, row: _menuObjsQuick.firstChild, nrow: 0 };
    this.updateTreeRow(data.quick, 0, x, false);
    while (x.row)
    {
      var node = x.row;
      x.row = node.nextSibling;
      x.root.removeChild(node);
    }

    x = { root: _menuObjsData, row: _menuObjsData.firstChild, nrow: 0 };
    this.updateTreeRow(data.tree, 0, x, true);
    while (x.row)
    {
      var node = x.row;
      x.row = node.nextSibling;
      x.root.removeChild(node);
    }

    // Update the search info.
    var search = data.def.search;
    var word = (search.pattern == '' ? " object" : " match");
    var plural = (search.pattern == '' ? "s" : "es");
    var searchres = (! search.valid ? "(invalid regexp)"
		     : (search.nmatches > 1 ? "(" + search.nmatches + word + plural + ")"
		        : search.nmatches == 1 ? "(1 " + word + ")"
		        : "(no " + word + plural + ")"));
    if (! _searchDelay && _menuObjsSearch.value != search.pattern)
      _menuObjsSearch.value = search.pattern;
    if (_menuObjsSearchRes.innerHTML != searchres)
      _menuObjsSearchRes.innerHTML = searchres;

    // Update the selectors.
    this.updateSelect(_menuObjsDataset, data.def.dataset, "/setDataset?n=");
    this.updateSelect(_menuObjsStep, data.def.step, "/setStep?n=");
    for (var i = _menuObjsFilter.firstChild, n = 0; i; i = i.nextSibling, ++n)
      if (i.value == data.def.filter)
      {
	_menuObjsFilter.selectedIndex = n;
	break;
      }
  }

  return this;
}();

//////////////////////////////////////////////////////////////////////
// Core objects: canvas.
GUI.Plugin.DQMCanvas = new function()
{
  var _SLIDER_UNIT	   = 20;
  var _IMG_BORDER_NONE     = "1px solid transparent";
  var _IMG_BORDER_NEW      = "1px solid #4a4";
  var _IMG_BORDER_ALARM    = "1px solid #a33";
  var _SIZES = [ { index: 0, label: "XS", className: "size-xsmall", width: 67,  height: 50 },
		 { index: 1, label: "S",  className: "size-small",  width: 133, height: 100 },
		 { index: 2, label: "M",  className: "size-medium", width: 266, height: 200 },
		 { index: 3, label: "L",  className: "size-large",  width: 532, height: 400 },
		 { index: 4, label: "XL", className: "size-xlarge", width: -1,  height: -1 } ];
  var _SIZEMAP = {}; _map(_SIZES, function(x) { _SIZEMAP[x.label] = x });
  var _AXES = { "lin": 0, "log": 1 };

  var _perpage		= 12;
  var _picked		= null;
  var _size		= "M";

  var _gui		= null;
  var _canvas		= this;
  var _gcanvas		= document.getElementById('canvas-area');
  var _optsarea		= document.getElementById('canvas-opts');
  var _menuarea		= document.getElementById('canvas-menu');
  var _playarea		= document.getElementById('canvas-play');
  var _playlist		= document.getElementById('canvas-play-list');
  var _helparea		= document.getElementById('canvas-help');

  var _szthumb		= null;
  var _szslider		= null;
  var _optPerPage	= null;
  var _optPlayMode	= null;
  var _optPlayInterval	= null;

  var _imgXType		= null;
  var _imgXMin		= null;
  var _imgXMax		= null;
  var _imgYType		= null;
  var _imgYMin		= null;
  var _imgYMax		= null;
  var _imgZType		= null;
  var _imgZMin		= null;
  var _imgZMax		= null;
  var _imgDrawOpts	= null;
  var _imgWithRef	= null;

  var _shown		= null;
  var _playInterval	= null;
  var _playinfo		= { on: 0, interval: 5, pos: 0, page: 0, max: 0 };
  var _click		= { event: null, timeout: null, timeClick: 0, timeDoubleClick: 0 };

  this.onresize = function()
  {
    var h = (_optsarea.style.display == '' ? _optsarea.clientHeight : _menuarea.clientHeight);
    var top = _gcanvas.offsetTop;
    var newtop = h+2;
    if (top != newtop)
      _gcanvas.style.top = newtop + 'px';

    var size = _SIZEMAP["XL"];
    for (var i = 0, obj = _gcanvas.firstChild; obj; ++i, obj = obj.nextSibling)
      if (obj.className.indexOf("size-xlarge") >= 0)
      {
        var nrows = obj.childNodes.length-1;
        for (var row = 0; row < nrows; ++row)
        {
	  var ncols = obj.childNodes[row+1].childNodes.length;
	  for (var col = 0; col < ncols; ++col)
	  {
	    var img = obj.childNodes[row+1].childNodes[col];
	    var suffix = this.setsize(img, size, row, nrows, col, ncols);
	    var url = img.src.replace(/;w=.*/, suffix);
	    if (img.src != url)
	    {
	      img.dqmsrc = url;
	      GUI.ImgLoader.load("div"+i+"."+row+"."+col, img, url);
	    }
	  }
        }
      }
  }

  this.describe = function(e)
  {
    e = e || window.event;
    var hit = (e.srcElement || e.target);
    if (hit.tagName == 'IMG' && hit.getAttribute('dqmhelp'))
      _helparea.innerHTML = "<div>" + hit.getAttribute('dqmhelp') + "</div>";
    else if (hit.tagName == 'IMG' && hit.title)
      _helparea.innerHTML = "<span style='color:#888'>(No description available for "
			    + hit.title + ".)</span>";
    else
      _helparea.innerHTML = "";
  }

  this.attach = function(gui)
  {
    _gui = gui;
    _gcanvas.style.bottom = '4em';
    if (! _optsarea.firstChild)
    {
      var div, item;
      _optsarea.appendChild(div = document.createElement('div'));
      div.className = 'group';

      div.appendChild(item = document.createElement('div'));
      item.style.cssFloat = 'left';
      item.style.styleFloat = 'left';
      item.style.marginRight = '6px';
      item.innerHTML = 'Small';

      div.appendChild(item = document.createElement('div'));
      item.id = 'canvas-size-slider';
      item.style.cssFloat = 'left';
      item.style.styleFloat = 'left';
      item.appendChild(item = document.createElement('div'));
      item.id = 'canvas-size-thumb';
      item.appendChild(item = document.createElement('img'));
      item.src = FULLROOTPATH + "/static/down.gif";
      item.width = "10";
      item.height = "10";

      div.appendChild(item = document.createElement('div'));
      item.style.cssFloat = 'left';
      item.style.styleFloat = 'left';
      item.style.marginLeft = '6px';
      item.style.marginRight = '6px';
      item.innerHTML = "Large";

      div.appendChild(item = document.createElement('div'));
      item.style.cssFloat = 'left';
      item.style.styleFloat = 'left';
      item.innerHTML = "<input style='font-size:100%' type='text' size='2'/> per page";
      _optPerPage = item.firstChild;
      _optPerPage.value = _perpage;
      _optPerPage.onchange = function() { _canvas.perpage(this.value); }

      _optsarea.appendChild(div = document.createElement('div'));
      div.className = 'group';

      div.appendChild(item = document.createElement('div'));
      item.style.cssFloat = 'left';
      item.style.styleFloat = 'left';
      item.innerHTML = "<a href='#'>Play</a><span>; rate </span><input style='font-size:100%' type='text' size='3' /><span> sec</span>";
      _optPlayMode = item.firstChild;
      _optPlayInterval = item.childNodes[2];
      _optPlayInterval.value = _playinfo.interval;
      _optPlayMode.onclick = function() { _canvas.play(); return false; }
      _optPlayInterval.onchange = function() { _canvas.playInterval(this.value); }

      _optsarea.appendChild(div = document.createElement('div'));
      div.className = 'group';

      div.appendChild(item = document.createElement('div'));
      item.style.cssFloat = 'left';
      item.style.styleFloat = 'left';
      item.innerHTML = "<a href='#'>Reset workspace</a>";
      item.firstChild.onclick = function() { _gui.makeCall(_url() + "/resetWorkspace"); return false; }
    }

    if (! _menuarea.firstChild)
    {
      _menuarea.innerHTML =
        "<div class='group'>X:"
        + " <select style='font-size:100%' id='m-obj-x-type'>"
        + "<option value='lin'>Linear</option>"
        + "<option value='log'>Log</option>"
        + "</select>"
        + " Range: <input style='font-size:100%' id='m-obj-x-min' size='4' type='text' />"
        + " ... <input style='font-size:100%' id='m-obj-x-max' size='4' type='text' /></div>"
        + "<div class='group'>Y:"
        + " <select style='font-size:100%' id='m-obj-y-type'>"
        + "<option value='lin'>Linear</option>"
        + "<option value='log'>Log</option>"
        + "</select>"
        + " Range: <input style='font-size:100%' id='m-obj-y-min' size='4' type='text' />"
        + " ... <input style='font-size:100%' id='m-obj-y-max' size='4' type='text' /></div>"

        + "<div class='group'>Z:"
        + " <select style='font-size:100%' id='m-obj-z-type'>"
        + "<option value='lin'>Linear</option>"
        + "<option value='log'>Log</option>"
        + "</select>"
        + " Range: <input style='font-size:100%' id='m-obj-z-min' size='4' type='text' />"
        + " ... <input style='font-size:100%' id='m-obj-z-max' size='4' type='text' /></div>"

        + "<div class='group'>Draw options: <input style='font-size:100%' id='m-obj-draw-opts' size='4' type='text' /></div>"

        + "<div class='group'>Reference: <input style='font-size:100%' id='m-obj-with-ref' type='checkbox' /></div>";

      _imgXType    = document.getElementById('m-obj-x-type');
      _imgXMin     = document.getElementById('m-obj-x-min');
      _imgXMax     = document.getElementById('m-obj-x-max');
      _imgYType    = document.getElementById('m-obj-y-type');
      _imgYMin     = document.getElementById('m-obj-y-min');
      _imgYMax     = document.getElementById('m-obj-y-max');
      _imgZType    = document.getElementById('m-obj-z-type');
      _imgZMin     = document.getElementById('m-obj-z-min');
      _imgZMax     = document.getElementById('m-obj-z-min');
      _imgDrawOpts = document.getElementById('m-obj-draw-opts');
      _imgWithRef  = document.getElementById('m-obj-with-ref');

      _imgXType.onchange    = function() { _canvas.alterImage("xtype",    this.value); return false; }
      _imgXMin.onchange     = function() { _canvas.alterImage("xmin",     this.value); return false; }
      _imgXMax.onchange     = function() { _canvas.alterImage("xmax",     this.value); return false; }
      _imgYType.onchange    = function() { _canvas.alterImage("ytype",    this.value); return false; }
      _imgYMin.onchange     = function() { _canvas.alterImage("ymin",     this.value); return false; }
      _imgYMax.onchange     = function() { _canvas.alterImage("ymax",     this.value); return false; }
      _imgZType.onchange    = function() { _canvas.alterImage("ztype",    this.value); return false; }
      _imgZMin.onchange     = function() { _canvas.alterImage("zmin",     this.value); return false; }
      _imgZMax.onchange     = function() { _canvas.alterImage("zmax",     this.value); return false; }
      _imgDrawOpts.onchange = function() { _canvas.alterImage("drawopts", this.value); return false; }
      _imgWithRef.onchange  = function() { _canvas.alterImage("withref",  this.checked ? "yes" : "no"); return false; }
    }

    if (! _szslider)
    {
      _szslider = document.getElementById("canvas-size-slider");
      _szthumb = document.getElementById("canvas-size-thumb");
      _szslider.onclick = function(e) { return _canvas.onSizeMove(e || window.event); };
    }

    this.setSizeSlider();

    // Install a click handler on canvas to deselect everything.
    // We want this to fire only if the click was actually on
    // the canvas itself, not on one of the items it contains.
    _gcanvas.onclick = _gcanvas.ondblclick = function(e) {
      return _dblclick(e || window.event, _click, _canvas.onPick, _canvas.onFocus); };
  }

  this.setSizeSlider = function() 
  {
    _szthumb.style.marginLeft = (_SIZEMAP[_size].index * _SLIDER_UNIT - 1) + "px";
  }

  this.detach = function()
  {
    if (_playInterval)
    {
      clearInterval(_playInterval);
      _playInterval = null;
    }

    _optsarea.style.display = 'none';
    _menuarea.style.display = 'none';
    _playarea.style.display = 'none';
    _helparea.style.display = 'none';
    _gcanvas.style.bottom = '0';
    while (_gcanvas.firstChild)
      _gcanvas.removeChild(_gcanvas.firstChild);
    _gcanvas.onclick = null;
  }

  this.onSizeMove = function(e)
  {
    if ((e.srcElement || e.target) != _szslider)
      return false;

    var pos = YAHOO.util.Dom.getXY(_szslider);
    var x = ((e.x != undefined && e.x != e.clientX)
	     ? e.x : (e.clientX - pos[0] - 12));
    if (isNaN(x))
      x = 2 * _SLIDER_UNIT;

    var index = Math.round(x / _SLIDER_UNIT);
    if (index < 0)
      index = 0;
    if (index > 4)
      index = 4;

    _szthumb.style.marginLeft = (index * _SLIDER_UNIT - 1) + "px";

    var size = _SIZES[index];
    if (_size != size.label)
    {
      _size = size.label; 
      _canvas.resize();
      _gui.makeCall(_url() + "/setSize?sz=" + size.label);
    }

    return false;
  }

  this.onPick = function(e)
  {
    var hit = (e.srcElement || e.target);
    if (hit == _gcanvas)
      _gui.makeCall(_url() + "/setPicked");
    else
    {
      while (hit.tagName != 'DIV' && hit.className.indexOf("item") >= 0)
	hit = hit.parentNode;
      _gui.makeCall(_url() + "/setPicked?n=" + encodeURIComponent(hit.title));
    }
    return false;
  }

  this.onFocus = function(e)
  {
    var hit = (e.srcElement || e.target);
    if (hit != _gcanvas)
    {
      while (hit.tagName != 'DIV' && hit.className.indexOf("item") >= 0)
	hit = hit.parentNode;
      if (hit && hit.title && _menuarea.style.display == 'none')
	_gui.makeCall(_url() + "/setFocus?n=" + encodeURIComponent(hit.title));
      else
	_gui.makeCall(_url() + "/setFocus");
    }
    return false;
  }

  this.updateOptions = function(params)
  {
    if (! params.focus)
    {
      if (_menuarea.style.display != 'none')
        _menuarea.style.display = 'none';

      if (_optsarea.style.display != '')
        _optsarea.style.display = '';

      _playinfo = params.play;

      if (_optPerPage.value != _perpage)
        _optPerPage.value = _perpage;

      var playmode = (_playinfo.on ? "Stop" : "Play");
      if (_optPlayMode.innerHTML != playmode)
        _optPlayMode.innerHTML = playmode;

      if (_optPlayInterval.value != _playinfo.interval)
        _optPlayInterval.value = _playinfo.interval;

      if (_playInterval)
      {
        clearInterval(_playInterval);
	_playInterval = null;
      }

      if (_playinfo.on)
      {
        _playInterval = setInterval(this.playStep, _playinfo.interval*1000);
	for (var i = 0; i < 3; ++i)
	  _optsarea.firstChild.childNodes[i].style.display = 'none';
      }
      else
      {
	for (var i = 0; i < 3; ++i)
	  _optsarea.firstChild.childNodes[i].style.display = '';
      }

      this.setSizeSlider();
    }
    else
    {
      if (_optsarea.style.display != 'none')
        _optsarea.style.display = 'none';
      if (_menuarea.style.display != '')
        _menuarea.style.display = '';

      var obj   = _gcanvas.firstChild;
      var opts  = null, ref  = null;
      var xtype = null, xmin = null, xmax = null;
      var ytype = null, ymin = null, ymax = null;
      var ztype = null, zmin = null, zmax = null;
      var nrows = obj.childNodes.length-1;
      for (var row = 0; row < nrows; ++row)
      {
	var ncols = obj.childNodes[row+1].childNodes.length;
	for (var col = 0; col < ncols; ++col)
	{
	  var img	= obj.childNodes[row+1].childNodes[col];
	  var url       = img.dqmsrc;
	  var iopts     = url.indexOf("drawopts=") >= 0 ? url.replace(/.*drawopts=([^;]*).*/, "$1") : "";
	  var iref      = url.indexOf("withref=") >= 0  ? url.replace(/.*withref=([^;]*).*/, "$1")  : "";
	  var ixtype    = url.indexOf("xtype=") >= 0    ? url.replace(/.*xtype=([^;]*).*/, "$1")    : "";
	  var ixmin     = url.indexOf("xmin=") >= 0     ? url.replace(/.*xmin=([^;]+).*/, "$1")     : "";
	  var ixmax     = url.indexOf("xmax=") >= 0     ? url.replace(/.*xmax=([^;]+).*/, "$1")     : "";
	  var iytype    = url.indexOf("ytype=") >= 0    ? url.replace(/.*ytype=([^;]*).*/, "$1")    : "";
	  var iymin     = url.indexOf("ymin=") >= 0     ? url.replace(/.*ymin=([^;]+).*/, "$1")     : "";
	  var iymax     = url.indexOf("ymax=") >= 0     ? url.replace(/.*ymax=([^;]+).*/, "$1")     : "";
	  var iztype    = url.indexOf("ztype=") >= 0    ? url.replace(/.*ztype=([^;]*).*/, "$1")    : "";
	  var izmin     = url.indexOf("zmin=") >= 0     ? url.replace(/.*zmin=([^;]+).*/, "$1")     : "";
	  var izmax     = url.indexOf("zmax=") >= 0     ? url.replace(/.*zmax=([^;]+).*/, "$1")     : "";

	  if (opts  == null) opts  = iopts;  else if (opts  != iopts)  opts  = "";
	  if (ref   == null) ref   = iref;   else if (ref   != iref)   ref   = "";
	  if (xtype == null) xtype = ixtype; else if (xtype != ixtype) xtype = "";
	  if (xmin  == null) xmin  = ixmin;  else if (xmin  != ixmin)  xmin  = "";
	  if (xmax  == null) xmax  = ixmax;  else if (xmax  != ixmax)  xmax  = "";
	  if (ytype == null) ytype = iytype; else if (ytype != iytype) ytype = "";
	  if (ymin  == null) ymin  = iymin;  else if (ymin  != iymin)  ymin  = "";
	  if (ymax  == null) ymax  = iymax;  else if (ymax  != iymax)  ymax  = "";
	  if (ztype == null) ztype = iztype; else if (ztype != iztype) ztype = "";
	  if (zmin  == null) zmin  = izmin;  else if (zmin  != izmin)  zmin  = "";
	  if (zmax  == null) zmax  = izmax;  else if (zmax  != izmax)  zmax  = "";
	}
      }

      if (opts  == null) opts  = "";
      if (ref   == null) ref   = "";
      if (xtype == null) xtype = "";
      if (xmin  == null) xmin  = "";
      if (xmax  == null) xmax  = "";
      if (ytype == null) ytype = "";
      if (ymin  == null) ymin  = "";
      if (ymax  == null) ymax  = "";
      if (ztype == null) ztype = "";
      if (zmin  == null) zmin  = "";
      if (zmax  == null) zmax  = "";

      if (ref == "yes")
        ref = "checked";
      else if (ref == "no")
        ref = "";

      if (_imgDrawOpts.value != opts)
        _imgDrawOpts.value = opts;

      if (_imgWithRef.checked != ref)
        _imgWithRef.value = ref;

      if (_imgXType.selectedIndex != _AXES[xtype])
        _imgXType.selectedIndex = _AXES[xtype];
      if (_imgXMin.value != xmin)
        _imgXMin.value = xmin;
      if (_imgXMax.value != xmax)
        _imgXMax.value = xmax;

      if (_imgYType.selectedIndex != _AXES[ytype])
        _imgYType.selectedIndex = _AXES[ytype];
      if (_imgYMin.value != ymin)
        _imgYMin.value = ymin;
      if (_imgYMax.value != ymax)
        _imgYMax.value = ymax;

      if (_imgZType.selectedIndex != _AXES[ztype])
        _imgZType.selectedIndex = _AXES[ztype];
      if (_imgZMin.value != zmin)
        _imgZMin.value = zmin;
      if (_imgZMax.value != zmax)
        _imgZMax.value = zmax;
    }
  }

  this.alterImage = function(param, value)
  {
    _gui.makeCall(_url() + "/alterImage?"
		  + "o=" + encodeURIComponent(_gcanvas.firstChild.title)
		  + ";opt=" + encodeURIComponent(param)
		  + ";value=" + encodeURIComponent(value));
    return false;
  }

  this.play = function()
  {
    if (_playInterval)
    {
      clearInterval(_playInterval);
      _playInterval = null;
    }

    if (_playinfo.on)
      _gui.makeCall(_url() + "/setPlayMode?mode=no");
    else
      _gui.makeCall(_url() + "/setPlayMode?mode=yes");
  }

  this.playInterval = function(value)
  {
    var newval = parseInt(value);
    if (isNaN(newval))
      newval = 5;
    else if (newval < 2)
      newval = 2;
    else if (newval > 99)
      newval = 99;

    if (newval != _playinfo.interval)
    {
      _playinfo.interval = newval;
      if (_optPlayInterval.value != newval)
        _optPlayInterval.value = newval;
      _gui.makeCall(_url() + "/setPlayInterval?v=" + newval);
    }
  }

  this.playStep = function()
  {
    if (! _playinfo.on)
      return;

    var curpos = _playinfo.pos - _playinfo.page*_perpage;
    if (curpos < _playlist.childNodes.length)
    {
      var curnode = _playlist.childNodes[curpos];
      _setclass(curnode, curnode.className.replace(/ playhead/, ""));
    }

    if (++_playinfo.pos >= _playinfo.max)
    {
      clearInterval(_playInterval);
      _playInterval = null;
      _playinfo.pos = 0;
      if (_playinfo.max)
      {
        _gui.makeCall(_url() + "/movePlayPosition?n=" + _playinfo.pos + ";move=yes");
        return;
      }
    }

    if (_shown.length)
    {
      curpos = _playinfo.pos - _playinfo.page*_perpage;
      if (curpos >= _shown.length && _shown.length)
      {
        clearInterval(_playInterval);
        _playInterval = null;
        _gui.makeCall(_url() + "/movePlayPosition?n=" + _playinfo.pos + ";move=yes");
      }
      else
      {
        curnode = _playlist.childNodes[curpos];
        _setclass(curnode, curnode.className + " playhead");

        _canvas.layout("div", _gcanvas, _gcanvas.firstChild, _shown[curpos], "XL", "", "div0")
        YAHOO.util.Connect.asyncRequest("GET", _url() + "/movePlayPosition?n=" + _playinfo.pos);
      }
    }
  }

  this.perpage = function(value)
  {
    var newval = parseInt(value);
    if (isNaN(newval))
      newval = 12;
    else if (newval < 1)
      newval = 1;
    else if (newval > 99)
      newval = 99;

    if (newval != _perpage)
    {
      if (_optPerPage.value != newval)
        _optPerPage.value = newval;
      _gui.makeCall(_url() + "/setPerPage?n=" + newval);
    }
  }

  this.pick = function(name)
  {
    _gui.makeCall(_url() + "/setPicked?n=" + encodeURIComponent(name));
    return false;
  }

  this.resize = function()
  {
    var size = _SIZEMAP[_size];
    for (var i = 0; i < _gcanvas.childNodes.length; ++i)
    {
      var obj = _gcanvas.childNodes[i];
      var title = obj.childNodes[0];
      var titlew = size.width + "px";
      if (size.width > 0 && title.style.width != titlew)
	title.style.width = titlew;
      else if (size < 0 && title.style.width != '')
	title.style.width = '';

      var nrows = obj.childNodes.length-1;
      for (var row = 0; row < nrows; ++row)
      {
	var ncols = obj.childNodes[row+1].childNodes.length;
	for (var col = 0; col < ncols; ++col)
	{
	  var img = obj.childNodes[row+1].childNodes[col];
	  var suffix = this.setsize(img, size, row, nrows, col, ncols);
	  var url = img.src.replace(/;w=.*/, suffix);
	  if (img.src != url)
	  {
	    img.dqmsrc = url;
	    GUI.ImgLoader.load("div"+i+"."+row+"."+col, img, url);
	  }
	}
      }

      var newClass = obj.className.replace(/size-[a-z]+/, size.className);
      if (obj.className != newClass)
	obj.className = newClass;
    }
  }

  this.setsize = function(img, size, row, rows, col, cols)
  {
    var maxwidth = _gcanvas.clientWidth - 50;
    var maxheight = _gcanvas.clientHeight - 70;
    if (maxwidth/1.33 > maxheight)
      maxwidth = Math.floor(1.33 * maxheight);
    else
      maxheight = Math.floor(maxwidth / 1.33);

    var width = (size.width > 0 ? size.width : maxwidth);
    var height = (size.height > 0 ? size.height : maxheight);
    var w = Math.round((width+2-2*cols)/cols); 
    var h = Math.round((height+2-2*rows)/rows);
    if (col == cols-1)
      w += (width+2) - (w+2)*cols;
    if (row == rows-1)
      h += (height+2) - (h+2)*rows;
    if (img.width != w)
      img.width = w;
    if (img.height != h)
      img.height = h;
    return ";w=" + w + ";h=" + h;
  }

  // Layout one canvas item.
  this.layout = function(type, container, item, obj, sz, xstyle, n)
  {
    var title;

    // Insert new object if one didn't exist previously.
    if (! item)
    {
      item = document.createElement(type);
      title = document.createElement("h3");
      item.appendChild(title);
      container.appendChild(item);
    }

    // Otherwise overwrite the existing one.
    else
    {
      title = item.firstChild;
    }

    // Now update the item.
    var alarm = 0;
    var picked = (_picked == obj.name);
    var dead = 0;
    for (var row = 0, nrows = obj.items.length; row < nrows; ++row)
      for (var col = 0, ncols = obj.items[row].length; col < ncols; ++col)
      {
	var ob = obj.items[row][col];
	alarm += ob.alarm;
	dead += ! ob.live;
      }

    var style = ("item"
		 + xstyle
		 + (alarm ? " alarm" : "")
		 + (picked ? " picked" : "")
		 + (dead ? " dead" : ""));
    _settitle(item, style, obj.name);
    _setval(title, "", obj.name.replace(/.*\//, ""));

    // Make sure layout in place conforms to the new structure.
    // Reinitialise and remove only where necessary, and reuse
    // old objects otherwise.  Give the image a URL to the
    // server call that generates the image, but don't touch
    // the URL unnecessarily.
    var size = _SIZEMAP[sz];
    var nrows = obj.items.length;
    for (var row = 0; row < nrows; ++row)
    {
      var rowdiv = (row < item.childNodes.length - 1
		    ? item.childNodes[row+1]
		    : document.createElement("div"));
      var rowparent = rowdiv.parentNode;
      var ncols = obj.items[row].length;

      for (var col = 0; col < ncols; ++col)
      {
	var img = (col < rowdiv.childNodes.length
		   ? rowdiv.childNodes[col]
		   : document.createElement("img"));
	if (! img.parentNode)
	  rowdiv.appendChild(img);
	var ob = obj.items[row][col];
	var sizeparam = this.setsize(img, size, row, nrows, col, ncols);
	var imgref = (! ob.name ? FULLROOTPATH + "/static/blank.gif?"
		      : FULLROOTPATH + "/plotfairy/"
		      + encodeURIComponent(ob.location).replace(/%2F/g, "/") + "/"
		      + encodeURIComponent(ob.name).replace(/%2F/g, "/") + "?");
	var param = "session=" + SESSION_ID
		    + ";v=" + ob.version
		    + ";xtype=" + ob.xaxis.type
		    + (ob.xaxis.min != null ? ";xmin=" + ob.xaxis.min : "")
		    + (ob.xaxis.max != null ? ";xmax=" + ob.xaxis.max : "")
		    + ";ytype=" + ob.yaxis.type
		    + (ob.yaxis.min != null ? ";ymin=" + ob.yaxis.min : "")
		    + (ob.yaxis.max != null ? ";ymax=" + ob.yaxis.max : "")
		    + ";ztype=" + ob.zaxis.type
		    + (ob.zaxis.min != null ? ";zmin=" + ob.zaxis.min : "")
		    + (ob.zaxis.max != null ? ";zmax=" + ob.zaxis.max : "")
		    + ";drawopts=" + ob.drawopts
		    + ";withref=" + ob.withref;
	var url = imgref + param + sizeparam;
	var border = ((ob.alarm && (nrows > 1 || ncols > 1))
		      ? _IMG_BORDER_ALARM : _IMG_BORDER_NONE);

        if (! ob.name && img.src != url)
	  img.src = url;
	else if (img.src != url)
	{
	  img.dqmsrc = url;
	  GUI.ImgLoader.load(n+"."+row+"."+col, img, url, border, _IMG_BORDER_NEW);
	  if (img.src.replace(/\?.*/, "?") != imgref)
	    img.src = FULLROOTPATH + "/static/blank.gif";
	  border = img.style.border;
	}
	if (img.style.border != border)
	  img.style.border = border;

	if (img.title != ob.name)
	  img.title = ob.name;

	if (img.getAttribute('dqmhelp') != ob.desc)
	  img.setAttribute('dqmhelp', ob.desc);

	if (! img.onmouseover)
	  img.onmouseover = this.describe;
      }

      while (rowdiv.childNodes.length > ncols)
	rowdiv.removeChild (rowdiv.lastChild);

      if (rowdiv.className != "row")
	rowdiv.className = "row";

      if (! rowparent)
	item.appendChild(rowdiv);
    }

    while (item.childNodes.length > nrows+1)
      item.removeChild (item.lastChild);

    if (item.className.indexOf("size-") < 0)
      item.className += " " + size.className;
    else
      item.className = item.className.replace(/size-[a-z]+/, size.className);

    var titlew = size.width + "px";
    if (size.width > 0 && title.style.width != titlew)
      title.style.width = titlew;
    else if (size.width < 0 && title.style.width != '')
      title.style.width = '';

    return item;
  }

  this.update = function(data)
  {
    _shown = data.items;
    _perpage = data.params.perpage;
    _picked = data.params.picked;
    _size = data.params.size;

    // Run a loop to handle new contents.  We modify the contents
    // of "_gcanvas" in place in order to avoid unpleasant flicker
    // and changes in scroll position.
    //
    // In the normal mode we fill the canvas with the shown objects.
    // In the play mode we fill the play list with small objects, and
    // the canvas with the currently selected object.
    var item = null;
    var playpos = data.params.play.pos - data.params.play.page * _perpage;
    var type = (data.params.play.on ? "td" : "div");
    var root = (data.params.play.on ? _playlist : _gcanvas);
    var size = (data.params.play.on ? "XS"
		: data.params.focus ? "XL"
		: data.params.size);

    for (var n = 0, item = root.firstChild; n < _shown.length;
	 ++n, item = item.nextSibling)
      item = this.layout(type, root, item, _shown[n], size,
			 n == playpos ? " playhead" : "",
			 type + n);

    // Remove whatever is left of old contents.
    while (item)
    {
      var next = item.nextSibling;
      root.removeChild(item);
      item = next;
    }

    // In play mode, put selected object as if in focus mode.
    if (data.params.play.on)
    {
      if (_playarea.style.display != '')
        _playarea.style.display = '';
      if (_helparea.style.display != 'none')
        _helparea.style.display = 'none';
      if (_gcanvas.style.bottom != '8.2em')
	_gcanvas.style.bottom = '8.2em';
      if (playpos < _shown.length)
      {
        item = this.layout("div", _gcanvas, _gcanvas.firstChild,
			   _shown[playpos], "XL", "", "div0");
	item = item.nextSibling;
      }
      else
        item = _gcanvas.firstChild;

      while (item)
      {
	var next = item.nextSibling;
	_gcanvas.removeChild(item);
	item = next;
      }
    }
    else
    {
      if (_playarea.style.display != 'none')
        _playarea.style.display = 'none';
      if (_helparea.style.display != '')
        _helparea.style.display = '';
      if (_gcanvas.style.bottom != '4em')
	_gcanvas.style.bottom = '4em';
    }

    // Update image options and page selector.
    this.updateOptions(data.params);
    this.onresize();
  }

  return this;
}();


//////////////////////////////////////////////////////////////////////
// Core objects: canvas.
GUI.Plugin.DQMSummary = new function()
{
  var _gui		   = null;
  var _canvas              = this;
  var _gcanvas             = document.getElementById('canvas-area');
  var _qrender		   = "";
  var _qavail		   = [];
  var _qmap		   = {};

  this.attach = function(gui)
  {
    _gui = gui;
    _gcanvas.style.bottom = 0;
  }

  this.detach = function()
  {
    while (_gcanvas.firstChild)
      _gcanvas.removeChild(_gcanvas.firstChild);
  }

  this.update = function(data)
  {
    _qrender = data.qrender;
    _qavail = [];
    _qmap = {};

    var status = data.items;
    var now = new Date();
    var content;

    if (! status.length)
    {
      content =
        "<div align='center' style='font-size: 2em; padding: 2em; color: #700'>"
	+ "<p>No new data are presently available.</p></div>";    
    }
    else
    {
      content = 
        "<div style='padding:.25em 2em'><table id='summary-table'"
	+ " cellpadding='2' style='white-space: nowrap; border-collapse: collapse'>"
	+ "<thead><tr bgcolor='#f0f0ff'>"
        + "<th valign='bottom' align='left'>Subsystem</th>"
        + "<th valign='bottom' align='right'>Summary</th>"
        + "<th valign='bottom' align='right'>Run</th>"
        + "<th valign='bottom' align='right'>Lumi<br />section</th>"
        + "<th valign='bottom' align='right'>Event</th>"
        + "<th valign='bottom' align='right'>Last update</th>"
        + "<th valign='bottom' align='right'>Last event</th>"
        + "<th valign='bottom' align='right'>Processed<br />events</th>"
        + "<th valign='bottom' align='right'>Event<br /> rate</th>"
        + "<th valign='bottom' align='right'>Monitor<br />elements</th>"
        // + "<th>CMSSW version</th>"
        // + "<th>DQM patch</th>"
        // + "<th>Host</th>"
        // + "<th>Process ID</th>"
        // + "<th>Working directory</th>"
        + "</tr></thead><tbody>";

      for (var i = 0, e = status.length; i < e; ++i)
      {
        var obj = status[i];
        var color = ((obj.reportSummary == null || obj.reportSummary < 0) ? "#000"
  		     : (obj.reportSummary < 0.85 ? "#a22"
	  		: (obj.reportSummary < 0.95 ? "#c30" : "#272")));
        var bcolor = ((obj.reportSummary == null || obj.reportSummary < 0) ? "#f0f0f0"
  		      : (obj.reportSummary < 0.85 ? "#fff0f0"
		         : (obj.reportSummary < 0.95 ? "#fff0c0" : "#efe")));

        var pts = new Date();
        var ets = new Date();

	// Event time stamp correct from CMSSW_1_7_1, DQMPATCH:04 onwards
        pts.setTime(obj.processTimeStamp*1000);
	ets.setTime((obj.eventTimeStamp & 0xffffffff) * 1000);

	content +=
          "<tr style='background-color:" + bcolor + "; color:" + color
	  + "'><td style='padding-left:3px"
	  + "; font-weight:" + (data.qplot == obj.processName ? "bold" : "normal")
	  + "'><a href='#' onclick='return GUI.Plugin.DQMSummary.qmap(this.textContent, true)'>"
	  + _sanitise(obj.processName) + "</a></td>"
          + "<td align='right'>"
          + (obj.reportSummary >= 0 ? sprintf("%.1f%%", obj.reportSummary*100.)
	     : (obj.reportSummary == -1 ? "No DAQ" : "N/A"))
          + "</td>"
          + "<td align='right'>" + obj.iRun + "</td>"
          + "<td align='right'>" + obj.iLumiSection + "</td>"
          + "<td align='right'>" + obj.iEvent + "</td>"
          + "<td align='right'>"
	  + (obj.processTimeStamp > 0 ? _formatTime(pts, now) : "(Never)") + "</td>"
          + "<td align='right'>" 
	  + (obj.eventTimeStamp > 10000 ? _formatTime(ets, now) : "(Never)") + "</td>"
          + "<td align='right'>" + _sanitise(obj.processedEvents) + "</td>"
          + "<td align='right'>"
	  + (obj.processEventRate != undefined
	     ? Math.round(obj.processEventRate*100.)/100.
	     : "&#8211;") + "</td>"
          + "<td style='padding-right:3px' align='right'>" + obj.MEs + "</td>"
          + "</tr>";

        _qavail.push(obj.processName);
        var q = _qmap[obj.processName] = {};
        for (key in obj)
	  if (key.substr(0, 22) == "reportSummaryContents/")
	    q[key.substring(22)] = obj[key];
        q["zzzstamp"] = obj.processTimeStamp;
      }

      content += "</tbody></table></div><div id='qdetail'></div>";
    }

    if (_gcanvas.innerHTML != content)
      _gcanvas.innerHTML = content;

    if (status.length)
      this.qmap(data.qplot, false);
  }

  this.qmap = function(label, reset)
  {
    var qkey = (label ? label : _qavail[0]);
    var qmap = _qmap[qkey] || {};
    var qkeys = []; for (var k in qmap) qkeys.push(k); qkeys = qkeys.sort();
    var smry = document.getElementById('summary-table');

    for (var i = 0, e = smry.rows.length; i < e; ++i)
    {
      var r = smry.rows[i];
      var sel = (r.cells[0].textContent == label);
      r.style.borderWidth = "1px";
      r.style.borderStyle = (sel ? "solid" : "none");
      r.style.borderColor = "#ccf";
      r.cells[0].style.fontWeight = (sel ? "bold" : "normal");
    }

    var content = "<div style='padding:1em 2em 0 0; float: left'>"
      + "<table cellpadding='10' style='margin-left:2em; border: none;"
      + " white-space: nowrap; border-collapse: collapse'>"
      + "<thead bgcolor='#f0f0ff'><tr><th>" + _sanitise(qkey)
      + " Part</th><th>Status</th></tr></thead>"
      + "<tbody>";

    var stamp = "";
    for (var i = 0, e = qkeys.length; i < e; ++i)
    {
      var name = qkeys[i];
      if (name == "zzzstamp")
      {
	stamp = ";stamp=" + qmap[name];
	continue;
      }
      var value = qmap[name] * 1.0;
      var color = (value < 0 ? "#000"
		   : (value < 0.85 ? "#a22"
		      : (value < 0.95 ? "#c30" : "#272")));
      var pcolor = (value < 0 ? '#f0f0f0'
		    : (value < 0.85 ? "#fff0f0"
		       : (value < 0.95 ? "#fff0c0" : "#efe")));
      content +=
	"<tr style='background-color:" + pcolor + ";color:" + color + "'>"
	+ "<td style='padding:0 .5em'>" + name + "</td>"
        + "<td align='right' style='padding:0 .5em'>"
        + (value >= 0 ? sprintf("%d%%", Math.round(value * 100))
	   : (value == -1 ? "No DAQ" : "N/A"))
	+ "</td></tr>";
    }
    content += "</tbody></table></div>";

    content += "<div id='smry-qplot' style='padding:1em .25em 0 0'>";
    content += this.qimage(qkey, stamp) + "</div>";
    document.getElementById("qdetail").innerHTML = content;

    if (reset)
      _gui.makeCall(_url() + "/setQualityPlot?n=" + encodeURIComponent(label));

    return false;
  }

  this.qimage = function(label, stamp)
  {
    var hasqplots = (_qavail.length > 0);
    if (label && _qmap[label])
    {
      var relurl = _qrender + "/" + label + "/EventInfo/reportSummaryMap";
      var url = FULLROOTPATH + "/plotfairy/"
	       + encodeURIComponent(relurl).replace(/%2F/g, "/")
	       + "?session=" + SESSION_ID + ";w=532;h=400" + stamp;
      return "<img src='" + url + "' title='" + label + " quality'"
	     + " width='532' height='400'/>";
    }
    else if (hasqplots)
      return "<span style='font-size:95%; color:#888'>(Click on subdetector"
	     + " to show quality data.)</span>";
    else
      return "<span style='font-size:95%; color:#888'>(No further quality"
	     + " data available.)</span>";
  }

  return this;
}();

//////////////////////////////////////////////////////////////////////
GUI.Plugin.DQMQuality = new function()
{
  var _gui		   = null;
  var _canvas              = this;
  var _gcanvas             = document.getElementById('canvas-area');
  var _qrender		   = "";
  var _qavail		   = [];
  var _qmap		   = {};

  this.attach = function(gui)
  {
    _gui = gui;
    _gcanvas.style.bottom = 0;
  }

  this.detach = function()
  {
    while (_gcanvas.firstChild)
      _gcanvas.removeChild(_gcanvas.firstChild);
  }

  this.update = function(data)
  {
    _qrender = data.qrender;
    _qavail = [];
    _qmap = {};

    var status = data.items;
    var now = new Date();
    var content = "";

    if (! status.length)
    {
      var content =
        "<div id='no-contents' align='center'"
	+ " style='font-size: 2em; padding: 2em; color: #700'>"
	+ "<p>No new data are presently available.</p></div>";

      if (_gcanvas.innerHTML != content)
        _gcanvas.innerHTML = content;
    }
    else
    {
      var now = new Date();
      var ets = new Date();
      var item = _gcanvas.firstChild;
      
      if (item && item.id == 'no-contents')
      {
        _gcanvas.removeChild(item);
        item = null;
      }

      for (var i = 0, e = status.length; i < e; ++i, item = item.nextSibling)
      {
        var title;
        if (! item)
	{
	  item = document.createElement("div");
	  title = document.createElement("h3");
	  item.appendChild(title);
	  _gcanvas.appendChild(item);
	}
	else
	{
	  title = item.firstChild;
	  img = title.nextSibling;
	  while (img && img.nextSibling)
	    item.removeChild(img.nextSibling);
	}

	var img = title.nextSibling;
	if (! img)
	{
	  img = document.createElement("img");
	  img.width = 266;
	  img.height = 200;
	  item.appendChild(img);
	}
	else
	  while (img.nextSibling)
	    item.removeChild(img.nextSibling);

        var obj = status[i];
        var color = ((obj.reportSummary == null || obj.reportSummary < 0) ? "#000"
  		     : (obj.reportSummary < 0.85 ? "#a22"
	  		: (obj.reportSummary < 0.95 ? "#c30" : "#272")));
        var bcolor = ((obj.reportSummary == null || obj.reportSummary < 0) ? "#f0f0f0"
  		      : (obj.reportSummary < 0.85 ? "#fff0f0"
		         : (obj.reportSummary < 0.95 ? "#fff0c0" : "#efe")));
	ets.setTime((obj.eventTimeStamp & 0xffffffff) * 1000);

	_settitle(item, "item size-medium", obj.label + " quality report");
	item.style.backgroundColor = bcolor;

	_setval(title, "",
		obj.label + " - "
		+ (obj.reportSummary >= 0 ? sprintf("%.1f%%", obj.reportSummary*100.)
		   : (obj.reportSummary == -1 ? "No DAQ" : "N/A"))
		+ " - " + (obj.eventTimeStamp > 10000 ? _formatTime(ets, now) : "(Never)"));
	title.style.color = color;

	var imgsrc =
	  FULLROOTPATH + "/plotfairy/"
	  + encodeURIComponent(obj.location).replace(/%2F/g, "/") + "/"
	  + encodeURIComponent(obj.name).replace(/%2F/g, "/") + "?"
	  + "session=" + SESSION_ID
	  + ";v=" + obj.version
	  + ";w=266;h=200";
	if (! img.src)
	  img.src = FULLROOTPATH + "/static/blank.gif";
	if (img.src != imgsrc)
	  GUI.ImgLoader.load('dqmq' + i, img, imgsrc);
      }

      while (_gcanvas.childNodes.length > status.length)
        _gcanvas.removeChild(_gcanvas.lastChild);
    }
  }

  return this;
}();
